Attribute VB_Name = "CM11XV2"
'****************************************************************************
'*                                                                          *
'*   CM11XV2 CM Visual Basic Library software, for use with CM110 or CM112  *
'*   Written in Microsoft Visual Basic 3.0 for Windows.                     *
'*                                                                          *
'****************************************************************************
'
'  Filename:  CM11XV2.BAS
'  Development Filename:  CM11XV1.BAS
'  Other Required Files:  All forms and other modules
'  Compiled Filename:  N/A (contents included to make exe based on 8-1209
'  File Rev: 2
'
'  Copyright CVI Laser Corporation, 1996
'
'
'  Revision History:
'
'     Date      Rev  By     Desc. of Change/Modification
'     --------  ---  ---    --------------------------------------
'     xx-xx-96  1    TT     Created in VB 3.0
'
'     06-04-97  2    TT     Upgrade to VB 4.0. The 16 bit version doesn't
'                           accept Dim Byte$. Chanded to Byte1$
'                           (See ~2 for new codes)
'
'
'
'****************************************************************************

Option Explicit
Const MB_ICONEXCLAMATION = 48   ' Warning message
Const MB_ICONSTOP = 16          ' Critical message

Dim ErrorStatus%    'Error: bit 7 is set; otherwise bit 7 is 0.
Dim StatusByte$     'status byte from CM11X
Dim CancelByte$     'CancelByte$ is "24"
Dim HighByte%, LowByte%

Function CMBaud%(Baud%, PortNo%, cmPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Sets the monochromator's RS232 baud rate and changes the Com port's
' baud rate to match.
'
' This function requires CMInitComPort() to be present.
'
' IMPORTANT!  The monochromator baud rate will change immediately after the
'             status and cancel bytes are sent.  The controlling program
'             must also switch baud rates to follow the change.  This
'             function performs that change.
'
'             The hand held controller (DK1200) will only communicate at
'             9600 baud.
'
'             The monochromator will remember the last baud rate when turned
'             off.  Any further communication must match this saved baud.
'
' Command: To Instrument:   <58><Baud Code Byte>
'          From Instrument: <Status Byte><24>
'
' This function's return value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' The passed parameters Baud%, PortNo%, and Timeout% are INTEGERs.
' cmPort is a commnunication control.
'
' Values of Baud% are 9600, 4800, 2400, 1200, 600, 300, 150, and 75.
'
' Valid choices for PortNo% are 1 = COM1, 2 = COM2, 3 = COM3, 4 = COM4.
'
' cmPort is the communications control assigned to the monochromator's
' COM port by the CMInitComPort%() function. It may be changed in this
' function when the Com port baud rate is changed.
'
  Dim BaudByte%, Status%
  On Error GoTo CMBaudErr
  ErrorStatus% = 0    'No error

  Select Case Baud%
    Case 9600
      BaudByte% = 0
    Case 4800
      BaudByte% = 1
    Case 2400
      BaudByte% = 2
    Case 1200
      BaudByte% = 3
    Case 600
      BaudByte% = 4
    Case 300
      BaudByte% = 5
    Case 150
      BaudByte% = 6
    Case 75
      BaudByte% = 7
    Case Else
      ErrorStatus% = 128      'Error; set bit 7 of return value
      MsgBox "The entered number is out of range", MB_ICONEXCLAMATION, "CM.BAS"
  End Select

  If ErrorStatus% = 0 Then
    cmPort.Output = Chr$(58) + Chr$(BaudByte%)
    ' Get Status and Cancel Bytes
    ErrorStatus% = CMGetStatusCancel%(cmPort, Timeout%)   'Timeout% in seconds

    ' Change the com port baud rate to match the monochromator
    Status% = CMInitComPort%(PortNo%, Baud%, cmPort)
    
    If Status% <> 0 Then ErrorStatus% = Status%
  End If

CMBaudResume:
  CMBaud% = ErrorStatus%
  Exit Function

CMBaudErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "CM.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume CMBaudResume

End Function

Function CMCalibrate%(wavelength&, cmPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Ajusts the monochromator calibration factor so the current wavelength
' position is defined as the wavelength sent with the command.
'
' CAUTION!  DO NOT calibrate the monochromator at very short wavelengths
'           (less than 100 nm.)  Always calibrate to a known reference
'           in the working region of the grating, away from 0 nm.
'
' Command: To Instrument:   <18><Wavelength High Byte><Wavelength Low Byte>
'          From Instrument: <Status Byte><24>
'
' This function's return value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' The passed parameter WaveLength& is a LONG INTEGER, and cmPort is a commnunication
' control.

'
' Wavelength& may range from 0 through 65535, but acceptable values to
' the monochromator will depend on the ruling of the grating selected and
' the current monochromator UNITS.
'
' cmPort is the communications control assigned to the monochromator's
' COM port by the CMInitComPort%() function.
'
  On Error GoTo CMCalibrateErr
  If (wavelength& >= 0) And (wavelength& <= 65535) Then
  'Calculate the 2 byte wavelength specifier to send in the command
    HighByte% = Int(wavelength& / 256!)
    LowByte% = Int(wavelength& - 256! * HighByte%)
    cmPort.Output = Chr$(18) + Chr$(HighByte%) + Chr$(LowByte%)
    'Get Status and Cancel Bytes
    ErrorStatus% = CMGetStatusCancel%(cmPort, Timeout%)   'Timeout% in seconds
  Else
    ErrorStatus% = 128      'Error; set bit 7 of return value
    MsgBox "The entered number is out of range", MB_ICONEXCLAMATION, "CM.BAS"
  End If

CMCalibrateResume:
  CMCalibrate% = ErrorStatus%
  Exit Function

CMCalibrateErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "CM.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume CMCalibrateResume

End Function

Function CMDecM1%(cmPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Decrements the zero offset value and grating angle by 0.0075 degrees
' in the clockwise direction for M1 (Entrance side grating turret
' in a CM112, or the only turret in a CM110.)
'
' NOTE:  This command is generally always followed by the ZERO command.
'        The monochromator does not track the change in wavelength incurred
'        by this command.  The new offset is not in effect unless the ZERO
'        command is issued.
'
' Command: To Instrument:   <01>
'          From Instrument: <Status Byte><24>
'
' This function's return value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' The passed parameter cmPort is a commnunication control.
'
' cmPort is the communications control assigned to the monochromator's
' COM port by the CMInitComPort%() function.
'
  On Error GoTo CMDecM1Err
  cmPort.Output = Chr$(1)
  'Get Status and Cancel Bytes
  ErrorStatus% = CMGetStatusCancel%(cmPort, Timeout%)

CMDecM1Resume:
  CMDecM1% = ErrorStatus%
  Exit Function

CMDecM1Err:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "CM.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume CMDecM1Resume

End Function

Function CMDecM2%(cmPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Decrements the zero offset value and grating angle by 0.0075 degrees
' in the clockwise direction for M2 (Exit side grating turret
' in a CM112, or the only turret in a CM110.)
'
' NOTE:  This command is generally always followed by the ZERO command.
'        The monochromator does not track the change in wavelength incurred
'        by this command.  The new offset is not in effect unless the ZERO
'        command is issued.
'
' Command: To Instrument:   <02>
'          From Instrument: <Status Byte><24>
'
' This function's return value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' The passed parameter cmPort is a commnunication control.
'
' cmPort is the communications control assigned to the monochromator's
' COM port by the CMInitComPort%() function.
'
  On Error GoTo CMDecM2Err
  cmPort.Output = Chr$(2)
  'Get Status and Cancel Bytes
  ErrorStatus% = CMGetStatusCancel%(cmPort, Timeout%)

CMDecM2Resume:
  CMDecM2% = ErrorStatus%
  Exit Function

CMDecM2Err:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "CM.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume CMDecM2Resume

End Function

Function CMEcho%(cmPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Verifies communication by sending a single byte and receiving that same
' byte back.
'
' Command: To Instrument:   <27>
'          From Instrument: <27>
'
' This function's return value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' The passed parameter cmPort is a control, and Timeout% as an integer.
'
' cmPort is the communications control assigned to the monochromator's
' COM port by the CMInitComPort%() function.
'
  Dim DelayTime%, Byte1$, Minutes%  ' ~2
  On Error GoTo CMEchoErr
  cmPort.Output = Chr$(27)
  ' Get Echo Byte
  If Timeout% > 0 Then
      ErrorStatus% = CMTimeout%(cmPort, Timeout%)
      If ErrorStatus% = 128 Then GoTo CMEchoResume
  End If
  ' ~2
  Byte1$ = cmPort.Input          'Read in byte
  If Asc(Byte1$) = 27 Then
    ErrorStatus% = 0            'no error
  Else
    ErrorStatus% = 128          'Error; set bit 7
  End If

CMEchoResume:
  CMEcho% = ErrorStatus%
  Exit Function

CMEchoErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "CM.BAS"
  ErrorStatus% = 128          'Error; set bit 7
  Resume CMEchoResume

End Function

Function CMGetStatusCancel%(cmPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Gets the status and cancel bytes sent back from the monochromator. This
' function is used by other monochromator library functions and is intended
' to be used elsewhere.
'
' This function returns an INTEGER that is value of the monochromator's
' status byte.
'
' cmPort is the communications control assigned to the monochromator's
' COM port by the CMInitComPort%() function.
'
'
  Dim DelayTime%, Minutes%, TimeoutErr%
  On Error GoTo CMGetStatusCancelErr
  ' Get Status Byte
  If Timeout% > 0 Then
    TimeoutErr% = CMTimeout%(cmPort, Timeout%)
    If TimeoutErr% = 128 Then
      ErrorStatus% = TimeoutErr%
      GoTo CMGetStatusCancelResume
    End If
  End If
  
  StatusByte$ = cmPort.Input              'Read in status byte
  ErrorStatus% = Asc(StatusByte$)         'Convert from char to integer

  ' Get Cancel Byte
  If Timeout% > 0 Then
    TimeoutErr% = CMTimeout%(cmPort, Timeout%)
    If TimeoutErr% = 128 Then
      ErrorStatus% = TimeoutErr%
      GoTo CMGetStatusCancelResume
    End If
  End If
  CancelByte$ = cmPort.Input              'Read in final byte
  If Asc(CancelByte$) <> 24 Then ErrorStatus% = 128

CMGetStatusCancelResume:
  CMGetStatusCancel% = ErrorStatus%
  Exit Function

CMGetStatusCancelErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "CM.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume CMGetStatusCancelResume

End Function

Function CMGoto%(wavelength&, cmPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Moves the monochromator to a desired wavelength, where the units of the
' wavelength are determined by the UNITS command.
'
' Command: To Instrument:   <16><Wavelength High Byte><Wavelength Low Byte>
'          From Instrument: <Status Byte><24>
'
' This function's return value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' The passed parameter WaveLength# is a LONG INTEGER and cmPort as commnunication
' control.
'
' Wavelength& may range from 0 through 65535, but acceptable values to
' the monochromator will depend on the ruling of the grating selected and
' the current monochromator UNITS.
'
' cmPort is the communications control assigned to the monochromator's
' COM port by the CMInitComPort%() function.
'
  Dim tmp&, Tmp1&, Tmp2&, tmp3&, tmp4&
  On Error GoTo CMGotoErr
  If (wavelength& >= 0) And (wavelength& <= 65535) Then

  ' Calculate the 2 byte wavelength specifier to send in the command
      
  HighByte% = Int(wavelength& / 256!)
  LowByte% = Int(wavelength& - 256! * HighByte%)
  
  cmPort.Output = Chr$(16)
  cmPort.CTSTimeout = 100   ' Sets the number of milliseconds to wait for the CTS signal
  cmPort.Output = Chr$(HighByte%) + Chr$(LowByte%)

  ' Get Status and Cancel Bytes
  ErrorStatus% = CMGetStatusCancel%(cmPort, Timeout%)   'Timeout% in seconds
  Else
    ErrorStatus% = 128      'Error; set bit 7 of return value
    MsgBox "The entered number is out of range", MB_ICONEXCLAMATION, "CM.BAS"
  End If

CMGotoResume:
  CMGoto% = ErrorStatus%
  Exit Function

CMGotoErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "CM.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume CMGotoResume

End Function

Function CMIncM1%(cmPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Increments the zero offset value and grating angle by 0.0075 degrees
' in the counterclockwise direction for M1 (Entrance side grating turret
' in a CM112, or the only turret in a CM110.)
'
' NOTE:  This command is generally always followed by the ZERO command.
'        The monochromator does not track the change in wavelength incurred
'        by this command.  The new offset is not in effect unless the ZERO
'        command is issued.
'
' Command: To Instrument:   <07>
'          From Instrument: <Status Byte><24>
'
' This function's return value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' The passed parameter cmPort is a commnunication control.
'
' cmPort is the communications control assigned to the monochromator's
' COM port by the CMInitComPort%() function.
'
  On Error GoTo CMIncM1Err
  cmPort.Output = Chr$(7)
  'Get Status and Cancel Bytes
  ErrorStatus% = CMGetStatusCancel%(cmPort, Timeout%)

CMIncM1Resume:
  CMIncM1% = ErrorStatus%
  Exit Function

CMIncM1Err:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "CM.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume CMIncM1Resume

End Function

Function CMIncM2%(cmPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Increments the zero offset value and grating angle by 0.0075 degrees
' in the counterclockwise direction for M2 (Exit side grating turret
' in a CM112, not applicable in a CM110.)
'
' NOTE:  This command is generally always followed by the ZERO command.
'        The monochromator does not track the change in wavelength incurred
'        by this command.  The new offset is not in effect unless the ZERO
'        command is issued.
'
' Command: To Instrument:   <08>
'          From Instrument: <Status Byte><24>
'
' This function's return value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' The passed parameter cmPort is a commnunication control.
'
' cmPort is the communications control assigned to the monochromator's
' COM port by the CMInitComPort%() function.
'
  On Error GoTo CMIncM2Err
  cmPort.Output = Chr$(8)
  'Get Status and Cancel Bytes
  ErrorStatus% = CMGetStatusCancel%(cmPort, Timeout%)

CMIncM2Resume:
  CMIncM2% = ErrorStatus%
  Exit Function

CMIncM2Err:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "CM.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume CMIncM2Resume

End Function

Function CMInitComPort%(PortNo%, Baud%, cmPort As Control)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Opens a RS232 com port to communicate with the monochromator.  8 data
' bits, 1 stop bit, and no parity are fixed.  Baud rate is selectable,
' as is the COM port to use.  The baud rate selected here must match the
' monochromator's baud rate (selected with the BAUD command.)
'
' This function must be run before any other monochromator library function
' calls.
'
' This function's return value is an INTEGER.  A value of zero indicates
' success.  Non-zero return values indicate an error has occured.
'
' The passed parameters PortNo%, and Baud% all INTEGERs. cmPort as a
' commnunications Control.
'
' Valid choices for PortNo% are 1 = COM1, 2 = COM2, 3 = COM3, 4 = COM4.
'
' Valid baud rates for CM110/112 are 9600, 4800, 2400, 1200, 600, 300, 150 and 75,
' but valid baud rates for Visual Basic are 9600, 4800, 2400, 1200, 600, 300.
'
  Dim InS$
  On Error GoTo CMInitComPortErr
  If cmPort.PortOpen Then
    cmPort.PortOpen = False
  End If
  cmPort.CommPort = PortNo%
  cmPort.Settings = Baud% & ",N,8,1"
  cmPort.RTSEnable = True             ' Enables the Request To Send line
  cmPort.InputLen = 1                 ' Read 1 character at the receive buffer. 0: entire contents.
  cmPort.PortOpen = True

  ErrorStatus% = 0                      ' No error

CMInitComPortResume:
  CMInitComPort% = ErrorStatus%
  Exit Function

CMInitComPortErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "CM.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume CMInitComPortResume

End Function

Function CMNovramRead%(Address%, DataByte%, cmPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Reads a byte from the monochromator's non-volatile memory.
'
' Command: To Instrument:   <156><Address Byte>
'          From Instrument: <Data Byte><Status Byte><24>
'
' cmPort as a communication control.
' The passed parameters Address%, Timeout% are intergers.
' DataByte% contains a returned byte.
'
' This function's returned value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' Valid choices for Address% are 0 through 127.
'
' cmPort is the communications control assigned to the monochromator's
' COM port by the CMInitComPort%() function.
'
  Dim ByteRd$
  On Error GoTo CMNovramReadErr
  If ((Address% >= 0) And (Address% <= 127)) Then
    cmPort.Output = Chr$(156) + Chr$(Address%)

    'Checking Timeout% for the first byte.
    If Timeout% > 0 Then
      ErrorStatus% = CMTimeout%(cmPort, Timeout%)
      If ErrorStatus% = 128 Then GoTo CMNovramReadResume
    End If
    'Get returned novram data
    While cmPort.InBufferCount = 0: Wend    'Wait until character present
    ByteRd$ = cmPort.Input            'Read in data byte
    DataByte% = Asc(ByteRd$)

    'Get Status and Cancel Bytes
    ErrorStatus% = CMGetStatusCancel%(cmPort, Timeout%)
  Else
    ErrorStatus% = 128
    MsgBox "The entered number is out of range", MB_ICONEXCLAMATION, "CM.BAS"
  End If

CMNovramReadResume:
  CMNovramRead% = ErrorStatus%
  Exit Function

CMNovramReadErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "CM.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume CMNovramReadResume

End Function

Function CMNovramWrite%(Address%, DataByte%, cmPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Writes a byte to the monochromator's non-volatile memory.
'
' WARNING!!!  Improper use of this command may corrupt the configuration
'             and calibration information of the monochromator.  DO NOT
'             use this command if you are unsure of the implications!
'
' Command: To Instrument:   <159><Address Byte><Data Byte><Checksum Byte>
'          From Instrument: <Status Byte><24>
'
' cmPort is a communication control.
' The passed parameters Address%, Timeout% are intergers.
' DataByte% contains a returned byte.
'
' This function's returned value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' Valid choices for Address% are 0 through 127.
'
' Valid values for DataByte% are 0 through 255.
'
' cmPort is the communications control assigned to the monochromator's
' COM port by the CMInitComPort%() function.
'
  Dim CheckSum%
  On Error GoTo CMNovramWriteErr
  If ((Address% >= 0) And (Address% <= 127) And (DataByte% >= 0) And (DataByte% <= 255)) Then
    CheckSum% = Address% + DataByte%
    If CheckSum% > 255 Then
      CheckSum% = CheckSum% - 256     'truncate to 1 byte long
    End If
    cmPort.Output = Chr$(159) + Chr$(Address%)
    cmPort.Output = Chr$(DataByte%) + Chr$(CheckSum%)

' Get Status and Cancel Bytes
    ErrorStatus% = CMGetStatusCancel%(cmPort, Timeout%)
  Else
    ErrorStatus% = 128      'Error; set bit 7 of return value
    MsgBox "The entered number is out of range", MB_ICONEXCLAMATION, "CM.BAS"
  End If

CMNovramWriteResume:
  CMNovramWrite% = ErrorStatus%
  Exit Function

CMNovramWriteErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "CM.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume CMNovramWriteResume

End Function

Function CMOrder%(Order%, cmPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Determines whether the grating rotates clockwise (positive order) or
' counterclockwise (negative order) for GOTO, STEP, and SCAN.
'
' Command: To Instrument:   <51><Order Byte>
'          From Instrument: <Status Byte><24>
'
' This function's return value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' The passed parameter Order% is an INTEGER.
'
' Values of Order% may be either 1 (for positive order) or 254 (for
' negative order.)
'
' cmPort is the communications control assigned to the monochromator's
' COM port by the CMInitComPort%() function.
'

  On Error GoTo CMOrderErr
  If (Order% = 254) Or (Order% = 1) Then
    cmPort.Output = Chr$(51) + Chr$(Order%)
' Get Status and Cancel Bytes
  ErrorStatus% = CMGetStatusCancel%(cmPort, Timeout%)
  Else
    ErrorStatus% = 128
    MsgBox "The entered number is out of range", MB_ICONEXCLAMATION, "CM.BAS"
  End If

CMOrderResume:
  CMOrder% = ErrorStatus%
  Exit Function

CMOrderErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "CM.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume CMOrderResume

End Function

Function CMQuery%(QueryCode%, DataByte&, cmPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Queries the monochromator for current setup and status information.
'
' Command: To Instrument:   <156><Query Code Byte>
'          From Instrument: <Data High Byte><Data Low Byte><Status Byte><24>
'
' This function's return value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' The passed parameters QueryCode% is INTEGER and cmPort as Control.  DataByte&
' is a LONG INTEGER.
'
' Valid choices for QueryCode% are 0 through 19, representing the following:
'
'   Query Code      Data Returned
'   ----------  -----------------------------------------------
'        0      Current wavelength
'        1      Monochromator type
'        2      Ruling of currently selected grating
'        3      Blaze wavelength of currently selected grating
'        4      Number of currently grating selected
'        5      Current scan speed
'        6      Current step size
'        7      Zero offset M1G1
'        8      Zero offset M1G2
'        9      Zero offset M2G1
'       10      Zero offset M2G2
'       11      Calibration constant G1
'       12      Calibration constant G1
'       13      Number of gratings installed
'       14      Current units selected
'       15      Grating ruling G1
'       16      Blaze wavelength G1
'       17      Grating ruling G2
'       18      Blaze wavelength G2
'       19      Serial number
'
' Refer to monochromator user manual for details on interpreting returned
' query data.
'
' cmPort is the communications control assigned to the monochromator's
' COM port by the CMInitComPort%() function.
'
  Dim DataHighByte$, DataLowByte$
  Dim High&, Low&
  On Error GoTo CMQueryErr
  If ((QueryCode% >= 0) And (QueryCode% <= 19)) Then
    cmPort.Output = Chr$(56) + Chr$(QueryCode%)
    'Checking Timeout% for the first byte.
    If Timeout% > 0 Then
      ErrorStatus% = CMTimeout%(cmPort, Timeout%)
      If ErrorStatus% = 128 Then GoTo CMQueryResume
    End If
    'Get returned query data
    DataHighByte$ = cmPort.Input            'Read in data byte
    High& = Asc(DataHighByte$)
    If Timeout% > 0 Then
      ErrorStatus% = CMTimeout%(cmPort, Timeout%)
      If ErrorStatus% = 128 Then GoTo CMQueryResume
    End If
    DataLowByte$ = cmPort.Input             'Read in data byte
    Low& = Asc(DataLowByte$)

    DataByte& = High& * 256 + Low&

    'Get Status and Cancel Bytes
    ErrorStatus% = CMGetStatusCancel%(cmPort, Timeout%)
  Else
    ErrorStatus% = 128      'Error; set bit 7 of return value
    MsgBox "The entered number is out of range", MB_ICONEXCLAMATION, "CM.BAS"
  End If

CMQueryResume:
  CMQuery% = ErrorStatus%
  Exit Function

CMQueryErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "CM.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume CMQueryResume

End Function

Function CMRtsCts%(cmPort As Control, RtsOutPc%, CtsInPc%)
' RtsOutPc% is true if the PC enables the RTS line, and vive-versa.
' ClsInPc% is true if CTS line is high, and vice-versa
  On Error GoTo CMRtsCtsErr
  ErrorStatus% = 0      ' No error.
  RtsOutPc% = cmPort.RTSEnable
  CtsInPc% = cmPort.CTSHolding

CMRtsCtsResume:
  CMRtsCts% = ErrorStatus%
  Exit Function

CMRtsCtsErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "CM.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume CMRtsCtsResume



End Function
Function CMReset%(cmPort As Control)
  On Error GoTo CMResetErr
  cmPort.Output = Chr$(255)
  cmPort.Output = Chr$(255)
  cmPort.Output = Chr$(255)
  ErrorStatus% = 0

CMResetResume:
  CMReset% = ErrorStatus%
  Exit Function

CMResetErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "CM.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume CMResetResume

End Function
Function CMScan%(StartWavelength&, EndWavelength&, cmPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Scans the monochromator between a start position and and end position at
' a rate determined by the SPEED command.  The start wavelength may be
' greater than the end wavelength or vice versa.
'
' Command: To Instrument:   <12>
'                           <Start Wavelength High Byte>
'                           <Start Wavelength Low Byte>
'                           <End Wavelength High Byte>
'                           <End Wavelength Low Byte>
'          From Instrument: <Status Byte><24>
'
' This function's return value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' The passed parameters StartWavelength& and EndWavelength& are LONG
' INTEGERs.
' cmPort is a communication control.
'
' Wavelengths may range from 0 through 65535, but acceptable values to
' the monochromator will depend on the ruling of the grating selected and
' the current monochromator UNITS.
'
' cmPort is the communications control assigned to the monochromator's
' COM port by the CMInitComPort%() function.
'
  Dim StartHighByte%, StartLowByte%, EndHighByte%, EndLowByte%
  On Error GoTo CMScanErr
  If (StartWavelength& >= 0) And (StartWavelength& <= 65535) And (EndWavelength& >= 0) And (EndWavelength& <= 65535) Then
    ' Calculate the two 2 byte wavelength specifiers to send in the command
    StartHighByte% = Int(StartWavelength& / 256!)
    StartLowByte% = Int(StartWavelength& - 256! * StartHighByte%)
   
    EndHighByte% = Int(EndWavelength& / 256!)
    EndLowByte% = Int(EndWavelength& - 256! * EndHighByte%)
  
    cmPort.Output = Chr$(12)
    cmPort.CTSTimeout = 100   ' Sets the number of milliseconds to wait for the CTS signal
    cmPort.Output = Chr$(StartHighByte%) + Chr$(StartLowByte%)
    cmPort.Output = Chr$(EndHighByte%) + Chr$(EndLowByte%)
  
    ' Get Status and Cancel Bytes
    ErrorStatus% = CMGetStatusCancel%(cmPort, Timeout%)
  Else
    ErrorStatus% = 128      'Error; set bit 7 of return value
    MsgBox "The entered number is out of range", MB_ICONEXCLAMATION, "CM.BAS"
  End If


CMScanResume:
  CMScan% = ErrorStatus%
  Exit Function

CMScanErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "CM.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume CMScanResume

End Function

Function CMSelect%(GratingNo%, cmPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Selects the grating that will be used.
'
' Command: To Instrument:   <26><Grating Byte>
'          From Instrument: <Status Byte><24>
'
' This function's return value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' The passed parameters GratingNo% as INTEGER, and cmPort as commnunication
' control.
'
' Valid values for GratingNo% are 1 and 2.
'
' cmPort is the communications control assigned to the monochromator's
' COM port by the CMInitComPort%() function.
'
  On Error GoTo CMSelectErr
  If (GratingNo% >= 1) And (GratingNo% <= 2) Then
    cmPort.Output = Chr$(26) + Chr$(GratingNo%)
    ' Get Status and Cancel Bytes
    ErrorStatus% = CMGetStatusCancel%(cmPort, Timeout%)
  Else
    ErrorStatus% = 128
    MsgBox "The entered number is out of range", MB_ICONEXCLAMATION, "CM.BAS"
  End If

CMSelectResume:
  CMSelect% = ErrorStatus%
  Exit Function

CMSelectErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "CM.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume CMSelectResume

End Function

Function CMSize%(StepSize%, cmPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Sets the magnitude and direction of the change in monochromator
' wavelength when using the STEP command.
'
' Command: To Instrument:   <55><Size Byte>
'          From Instrument: <Status Byte><24>
'
' This function's return value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' The passed parameters StepSize% is an INTEGER.
'
' Values of StepSize% may be from -127 to +127 excluding 0, and are in
' the units selected in the monochromator (set by the UNITS command.)
' Positive numbers represent UP (move to higher wavelength), and negative
' numbers represent DOWN (move to lower wavelength).
'
' cmPort is the communications control assigned to the monochromator's
' COM port by the CMInitComPort%() function.
'
  On Error GoTo CMSizeErr
  If (StepSize% >= -127) And (StepSize% <= 127) Then
    If StepSize% < 0 Then StepSize% = Abs(StepSize%) + 128
    cmPort.Output = Chr$(55) + Chr$(StepSize%)

    'Get Status and Cancel Bytes
    ErrorStatus% = CMGetStatusCancel%(cmPort, Timeout%)

  Else
    ErrorStatus% = 128
    MsgBox "The entered number is out of range", MB_ICONEXCLAMATION, "CM.BAS"
  End If

CMSizeResume:
  CMSize% = ErrorStatus%
  Exit Function

CMSizeErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "CM.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume CMSizeResume

End Function

Function CMSpeed%(ScanSpeed%, cmPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Sets the monochromator's scan speed (for use with the SCAN command.)
'
' Command: To Instrument:   <13><Speed High Byte><Speed Low Byte>
'          From Instrument: <Status Byte><24>
'
' This function's return value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' cmPort is a communication control.
' The passed parameters ScanSpeed% and Timeout% are both INTEGERs.
'
' ScanSpeed% is always in Angstroms per second, regardless of the UNITS
' selected in the monochromator.
'
' Valid values of ScanSpeed% depend on the ruling of the grating selected,
' per the following table:
'
'  Grating    Allowed Scan Speeds [Angstroms per second]
'  [grv/mm]
'  --------   ---------------------------------------------------
'   3600        333   166    83    41   20   10    5    2   1   0
'   2400        500   250   125    62   31   15    7    3   1   0
'   1800        666   332   166    82   40   20   10    4   2   0
'   1200       1000   500   250   125   62   31   15    7   3   1
'    600       2000  1000   500   250  124   62   30   14   6   2
'    300       4000  2000  1000   500  248  124   60   28  12   4
'    150       8000  4000  2000  1000  496  248  120   56  24   8
'     75      16000  8000  4000  2000  992  496  240  112  48  16
'
' cmPort is the communications control assigned to the monochromator's
' COM port by the CMInitComPort%() function.
'
'  Calculate the 2 byte speed specifier to send in the command
'
  On Error GoTo CMSpeedErr
  HighByte% = Int(ScanSpeed% / 256!)
  LowByte% = Int(ScanSpeed% - 256! * HighByte%)
  cmPort.Output = Chr$(13) + Chr$(HighByte%) + Chr$(LowByte%)
  ' Get Status and Cancel Bytes
  ErrorStatus% = CMGetStatusCancel%(cmPort, Timeout%)

CMSpeedResume:
  CMSpeed% = ErrorStatus%
  Exit Function

CMSpeedErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "CM.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume CMSpeedResume

End Function

Function CMStep%(cmPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Changes the monochromator's wavelength by an amount defined by the
' SIZE command.
'
' Command: To Instrument:   <54>
'          From Instrument: <Status Byte><24>
'
' This function's return value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' The passed parameter FileHandle% is an INTEGER.
'
' cmPort is the communications control assigned to the monochromator's
' COM port by the CMInitComPort%() function.
'
  On Error GoTo CMStepErr
  cmPort.Output = Chr$(54)
  
  'Get Status and Cancel Bytes
  ErrorStatus% = CMGetStatusCancel%(cmPort, Timeout%)

CMStepResume:
  CMStep% = ErrorStatus%
  Exit Function

CMStepErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "CM.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume CMStepResume

End Function

Function CMTimeout%(cmPort As Control, Timeout%)

' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' This function will return 128 (bit 7 sets 1) if during the time was set
' by Timeout% but no byte was received on the transmission line. Otherwise
' the return number is 0.
'
' cmPort is the communications control assigned to the monochromators
' COM port by the CMInitComPort%() function.
'
  Dim Minutes%, DelayTime%
  If Timeout% > 0 Then
    If Timeout% > 3599 Then Timeout% = 3599
    Minutes% = Minute(Time$)
    DelayTime% = Second(Time$) + Timeout%     'Timeout% in seconds
    While (cmPort.InBufferCount = 0) And ((Minute(Time$) - Minutes%) * 60 + Second(Time$) < DelayTime%)
    Wend    'Wait until character present or waiting time is less than delay time
    If cmPort.InBufferCount = 0 Then
      'MsgBox "Errors in the serial commnuncation line", MB_ICONSTOP, "CM.BAS"
      CMTimeout% = 128
    Else
      CMTimeout% = 0
    End If
  End If
End Function

Function CMType%(TypeCode%, cmPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Configures a CM112 monochromator to operate in additive or subtractive
' dispersion mode.  This command is not applicable for a CM110.
'
' Command: To Instrument:   <57><Type Code Byte>
'          From Instrument: <Status Byte><24>
'
' This function's return value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' The passed parameters TypeCode% is INTEGER.
'
' Values of TypeCode% are as follows:
'
'    TypeCode%    Configuration
'    ---------    ------------------------------
'        1        Additive Double (CM112 only)
'       254       Subtractive Double (CM112 only)
'
' cmPort is the communications control assigned to the monochromator's
' COM port by the CMInitComPort%() function.
'
  On Error GoTo CMTypeErr
  If (TypeCode% = 1) Or (TypeCode% = 254) Then
    cmPort.Output = Chr$(57) + Chr$(TypeCode%)
    ' Get Status and Cancel Bytes
    ErrorStatus% = CMGetStatusCancel%(cmPort, Timeout%)
  Else
    ErrorStatus% = 128      'Error; set bit 7 of return value
    MsgBox "The entered number is out of range", MB_ICONEXCLAMATION, "CM.BAS"
  End If

CMTypeResume:
  CMType% = ErrorStatus%
  Exit Function

CMTypeErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "CM.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume CMTypeResume

End Function

Function CMUnits%(Units%, cmPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Sets the units currently used in the monochromator by the GOTO, SCAN,
' SIZE, and CALIBRATE commands.
'
' Command: To Instrument:   <50><Units Code Byte>
'          From Instrument: <Status Byte><24>
'
' This function's return value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' The passed parameters Units% INTEGER.
'
' Values of Units% are as follows:
'
'    Units%    Wavelength Units
'    ------    ----------------
'      0         Microns
'      1         Nanometers
'      2         Angstroms
'
' cmPort is the communications control assigned to the monochromator's
' COM port by the CMInitComPort%() function.
'
  On Error GoTo CMUnitsErr
  If (Units% >= 0) And (Units% <= 2) Then
    cmPort.Output = Chr$(50) + Chr$(Units%)
    'Get Status and Cancel Bytes
    ErrorStatus% = CMGetStatusCancel%(cmPort, Timeout%)
  Else
    ErrorStatus% = 128      'Error; set bit 7 of return value
    MsgBox "The entered number is out of range", MB_ICONEXCLAMATION, "CM.BAS"
  End If

CMUnitsResume:
  CMUnits% = ErrorStatus%
  Exit Function

CMUnitsErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "CM.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume CMUnitsResume

End Function

Function CMZero%(cmPort As Control, Timeout%)
' To disable Timeout% (in seconds), set Timeout% to 0
' Timeout% will be set to 1hr if >1hr.
'
' Ajusts the monochromator zero offset factors so the current position,
' reached by using INC and DEC only, is defined as the zero order
' transmission wavelength (ie: 0 nm).
'
' IMPORTANT!  The positon prior to sending the ZERO command must be obtained
'             using the INC and DEC commands following a GOTO 0.
'
' Command: To Instrument:   <52><01>
'          From Instrument: <Status Byte><24>
'
' This function's return value is an INTEGER.  A value of zero at bit 7
' (the MSB) indicates success.
'
' cmPort is the communications control assigned to the monochromator's
' COM port by the CMInitComPort%() function.
'
  On Error GoTo CMZeroErr
  cmPort.Output = Chr$(52) + Chr$(1)
  'Get Status and Cancel Bytes
  ErrorStatus% = CMGetStatusCancel%(cmPort, Timeout%)

CMZeroResume:
  CMZero% = ErrorStatus%
  Exit Function

CMZeroErr:
  MsgBox "Error code is : " & Err, MB_ICONEXCLAMATION, "CM.BAS"
  ErrorStatus% = 128      'Error; set bit 7 of return value
  Resume CMZeroResume

End Function

